/*
  adf2xdf.c Copyright (c) 2003 Karsten Sperling
*/

#include <stdio.h>
#include <string.h>

#define XON  0x11
#define XOFF 0x13
#define ESC  0x1b

int main(int argc, char **argv)
{
    FILE *input, *output;
    unsigned char inbuf[1024], outbuf[sizeof(inbuf)*2];
    unsigned char *inp, *outp;
    int read, i;
    
    if (argc != 2 && argc != 3)
    {
        fprintf(stderr, "usage: adf2xdf <input> [<output>]\n");
        return 2;
    }
    
    /* open input file */
    input = fopen(argv[1], "rb");
    if (!input)
    {
        fprintf(stderr, "failed to open '%s' for reading\n", argv[1]);
        return 1;
    }
    
    /* if no output file is given, use <input>.xdf */
    if (argc == 3) outp = argv[2];
    else
    {
        strncpy(outbuf, argv[1], sizeof(outbuf));
        outbuf[sizeof(outbuf)-5] = 0;
        outp = strcat(outbuf, ".xdf");
    }
    
    /* open output file */
    output = fopen(outp, "wb");
    if (!output)
    {
        fclose(input);
        fprintf(stderr, "failed to open '%s' for writing\n", argv[2]);
        return 1;
    }
    
    /* encode */
    while (1)
    {
        read = fread(inbuf, 1, sizeof(inbuf), input);
        if (read == 0) break;
        
        inp  = inbuf;
        outp = outbuf;
        for (i=0; i<read; i++)
        {
            unsigned char b = *(inp++);
            switch (b)
            {
            case XON:
                *(outp++) = ESC;
                *(outp++) = 0;
                break;
            case XOFF:
                *(outp++) = ESC;
                *(outp++) = 1;
                break;
            case ESC:
                *(outp++) = ESC;
                *(outp++) = ESC;
                break;
            default:
                *(outp++) = b;
            }
        }
        
        fwrite(outbuf, 1, outp-outbuf, output);
    }
    
    /* add 2k of padding */
    for (i=0; i<sizeof(outbuf); i++) outbuf[i] = 0;
    fwrite(outbuf, 1, sizeof(outbuf), output);
    
    /* done */
    fclose(input);
    fclose(output);
    return 0;
}
